#include <efi.h>
#include <efilib.h>

EFI_STATUS
efi_main (EFI_HANDLE image, EFI_SYSTEM_TABLE *systab)
{
	SIMPLE_TEXT_OUTPUT_INTERFACE *conout;

        InitializeLib(image, systab);
	conout = systab->ConOut;
	uefi_call_wrapper(conout->OutputString, 2, conout, u"Hello World!\r\n");

	return EFI_SUCCESS;
}
